/*
 * PARSE.C
*/

#include "env.h"  /* must appear before first C statement */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "hash.h"
#include "dmio.h"
#include "tran.h"

#if defined(IBM_370_OFFSETOF_BUG)
#undef FIELDOFFSET
#undef ARRAYOFFSET
#define dclProto(type)	static type Proto_##type
#define FIELDOFFSET(type, field)    \
	((size_t)  (((char *) &Proto_##type.field) - ((char *) &Proto_##type)))
#define ARRAYOFFSET(type, field)    \
	((size_t)  (((char *) Proto_##type.field) - ((char *) &Proto_##type)))

dclProto(attribute);
#endif

static void cfStoreOn(char *pc, char *pcf);
static void strStoreOn(char *pc, char *pstr);
static void tokStoreOn(char *pc, char *ptok);
static void ulongStoreOn(char *pc, ulong *pul);
static void relopStoreOn(char *pc, relop *pul);

static argumentInfo aiDecimalPlacesArgs[] =
{
	integerArgType,   FIELDOFFSET(attribute, u.fd.decimalPlaces),	0
};
#define aiDecimalPlacesArgCt	\
	(sizeof(aiDecimalPlacesArgs) / sizeof(argumentInfo))

static argumentInfo aiIndirectKeyedStringArgs[] =
{
	cobolFieldArgType,	ARRAYOFFSET(attribute, u.iks.keyfield),	MAX_COBOL_NAME,
	stringArgType,			ARRAYOFFSET(attribute, u.iks.filename),	MAX_FILENAME_LEN
};
#define aiIndirectKeyedStringArgCt	\
	(sizeof(aiIndirectKeyedStringArgs) / sizeof(argumentInfo))

static argumentInfo aiKeyedStringArgs[] =
{
	tokenArgType,		ARRAYOFFSET(attribute, u.ks.keyvalue),	MAX_TOKEN_LEN,
	stringArgType,		ARRAYOFFSET(attribute, u.ks.filename),	MAX_FILENAME_LEN
};
#define aiKeyedStringArgCt	\
	(sizeof(aiKeyedStringArgs) / sizeof(argumentInfo))

static argumentInfo aiTableArgs[] =
{
	tokenArgType,	ARRAYOFFSET(attribute, u.tb.tblName),	MAX_TOKEN_LEN,
	stringArgType,	ARRAYOFFSET(attribute, u.tb.strElse),	MAX_COBOL_NAME
};
#define aiTableArgCt	\
	(sizeof(aiTableArgs) / sizeof(argumentInfo))

static argumentInfo aiKeyedStringIfArgs[] =
{
	tokenArgType,	ARRAYOFFSET(attribute, u.ksif.keyvalue),	MAX_TOKEN_LEN,
	relopArgType,	FIELDOFFSET(attribute, u.ksif.r),			0,
	stringArgType,	ARRAYOFFSET(attribute, u.ksif.value),		MAX_COBOL_NAME,
	stringArgType,	ARRAYOFFSET(attribute, u.ksif.strElse),	MAX_COBOL_NAME,
	stringArgType,	ARRAYOFFSET(attribute, u.ksif.filename),	MAX_FILENAME_LEN
};
#define aiKeyedStringIfArgCt	\
	(sizeof(aiKeyedStringIfArgs) / sizeof(argumentInfo))

static argumentInfo aiKeyedStringTableArgs[] =
{
	tokenArgType,		ARRAYOFFSET(attribute, u.kstab.keyvalue),	MAX_TOKEN_LEN,
	tokenArgType,		ARRAYOFFSET(attribute, u.kstab.tblName),	MAX_TOKEN_LEN,
	stringArgType,	   ARRAYOFFSET(attribute, u.kstab.strElse),	MAX_COBOL_NAME,
	stringArgType,		ARRAYOFFSET(attribute, u.kstab.filename),	MAX_FILENAME_LEN
};
#define aiKeyedStringTableArgCt	\
	(sizeof(aiKeyedStringTableArgs) / sizeof(argumentInfo))

static argumentInfo aiBy12Args[] =
{
	cobolFieldArgType,	ARRAYOFFSET(attribute, u.by12.field),	MAX_COBOL_NAME
};
#define aiBy12ArgCt	\
	(sizeof(aiBy12Args) / sizeof(argumentInfo))

static argumentInfo aiBy12OffArgs[] =
{
	integerArgType,	FIELDOFFSET(attribute, u.by12Off.ulOff),		0,
	integerArgType,	FIELDOFFSET(attribute, u.by12Off.ulLen),		0
};
#define aiBy12OffArgCt	\
	(sizeof(aiBy12OffArgs) / sizeof(argumentInfo))

static argumentInfo aiKeyedStringIf2Args[] =
{
	tokenArgType,	ARRAYOFFSET(attribute, u.ksif2.keyvalue),	MAX_TOKEN_LEN,
	relopArgType,	FIELDOFFSET(attribute, u.ksif2.r),			0,
	stringArgType,	ARRAYOFFSET(attribute, u.ksif2.value),		MAX_COBOL_NAME,
	stringArgType,	ARRAYOFFSET(attribute, u.ksif2.filename),	MAX_FILENAME_LEN
};
#define aiKeyedStringIf2ArgCt	\
	(sizeof(aiKeyedStringIf2Args) / sizeof(argumentInfo))

static argumentInfo aiKeyedStringIfIntArgs[] =
{
	tokenArgType,	ARRAYOFFSET(attribute, u.ksifInt.keyvalue),	MAX_TOKEN_LEN,
	relopArgType,	FIELDOFFSET(attribute, u.ksifInt.r),			0,
	integerArgType,FIELDOFFSET(attribute, u.ksifInt.value),		0,
	stringArgType,	ARRAYOFFSET(attribute, u.ksifInt.strElse),	MAX_COBOL_NAME,
	stringArgType,	ARRAYOFFSET(attribute, u.ksifInt.filename),	MAX_FILENAME_LEN
};
#define aiKeyedStringIfIntArgCt	\
	(sizeof(aiKeyedStringIfIntArgs) / sizeof(argumentInfo))

static argumentInfo aiValidateArgs[] =
{
	tokenArgType,	ARRAYOFFSET(attribute, u.val.tblName),	MAX_TOKEN_LEN,
	stringArgType,	ARRAYOFFSET(attribute, u.val.strElse),	MAX_COBOL_NAME
};
#define aiValidateArgCt	\
	(sizeof(aiValidateArgs) / sizeof(argumentInfo))

static argumentInfo aiValidateIndirectArgs[] =
{
	tokenArgType,			ARRAYOFFSET(attribute, u.valind.tblName),
		MAX_TOKEN_LEN,
	cobolFieldArgType,	ARRAYOFFSET(attribute, u.valind.field),
		MAX_COBOL_NAME,
	stringArgType,			ARRAYOFFSET(attribute, u.valind.strElse),
		MAX_COBOL_NAME
};
#define aiValidateIndirectArgCt	\
	(sizeof(aiValidateIndirectArgs) / sizeof(argumentInfo))

static argumentInfo aiValidateIndirectOffArgs[] =
{
	tokenArgType,		ARRAYOFFSET(attribute, u.valindOff.tblName),
		MAX_TOKEN_LEN,
	integerArgType,	FIELDOFFSET(attribute, u.valindOff.ulOffset),
		0,
	integerArgType,   FIELDOFFSET(attribute, u.valindOff.ulLen),
		0,
	stringArgType,		ARRAYOFFSET(attribute, u.valindOff.strElse),
		MAX_COBOL_NAME
};
#define aiValidateIndirectOffArgCt	\
	(sizeof(aiValidateIndirectOffArgs) / sizeof(argumentInfo))

/* indexed by argumentType */
typedef void (*pfPcPv)(char *pc, void *pv);
static pfPcPv funcStoreOn[] =
{
	/* cobolField */	cfStoreOn,
	/* integer 	  */	ulongStoreOn,
	/* string  	  */	strStoreOn,
	/* token  	  */	tokStoreOn,
	/* relop  	  */	relopStoreOn
};

static attributeDesc adTable[] =
{
	"by12",								by12Attribute,
		aiBy12ArgCt,							aiBy12Args,
	"by12Off",							by12OffAttribute,
		aiBy12OffArgCt,						aiBy12OffArgs,
	"byk",								byThousandAttribute,
		0,											NULL,
	"datecon",							dateAttribute,
		0,											NULL,
	"decimalplaces",					decimalPlacesAttribute,
		aiDecimalPlacesArgCt,				aiDecimalPlacesArgs,
	"indirectkeyedstring",			indirectKeyedStringAttribute,
		aiIndirectKeyedStringArgCt,		aiIndirectKeyedStringArgs,
	"key",								keyAttribute,
		0,											NULL,
	"keyedstring",						keyedStringAttribute,
		aiKeyedStringArgCt,					aiKeyedStringArgs,
	"keyedstringif",					keyedStringIfAttribute,
		aiKeyedStringIfArgCt,				aiKeyedStringIfArgs,
	"keyedstringif2",					keyedStringIf2Attribute,
		aiKeyedStringIf2ArgCt,				aiKeyedStringIf2Args,
	"keyedstringifint",				keyedStringIfIntAttribute,
		aiKeyedStringIfIntArgCt,			aiKeyedStringIfIntArgs,
	"keyedstringtable",				keyedStringTableAttribute,
		aiKeyedStringTableArgCt,			aiKeyedStringTableArgs,
	"tablecon",          			tableAttribute,
		aiTableArgCt,							aiTableArgs,
	"tomonths",							toMonthsAttribute,
		0,											NULL,
	"validate",							validateAttribute,
		aiValidateArgCt,                aiValidateArgs,
	"validateindirect",				validateIndirectAttribute,
		aiValidateIndirectArgCt,			aiValidateIndirectArgs,
	"validateindirectOff",			validateIndirectOffAttribute,
		aiValidateIndirectOffArgCt,	aiValidateIndirectOffArgs,
	"unknown",							unknownAttribute,
		0,										NULL,
	""
};

typedef struct
{
	char *relopName;
	relop r;
} relopData;

/* indexed by relop */
static relopData relopTable[] =
{
	"=",		eqRelop,
	"!=",		neqRelop,
	"",		unknownRelop
};

bool getDecimalNumber(char **ppc, unsigned long *pulNum)
{
	bool rc = false;
	char *p = *ppc;
	char strDecimalNumber[MAX_DEC_DIGITS + 1];
	size_t sIdx = 0;

	if(NULL == p)
		return(false);

	/* skip white space */
	while(*p && isspace(*p))
		p++;

	while(*p && isdigit(*p))
	{
		if(sIdx < MAX_DEC_DIGITS)
		{
			rc = true;
			strDecimalNumber[sIdx++] = *p++;
		}
		else
		{
			strDecimalNumber[sIdx] = EOS;
			fprintf(stderr, "decimal number too long: '%s' (truncated)\n",
				strDecimalNumber);
			return(false);
		}
	}
	if(rc)
	{
		strDecimalNumber[sIdx] = EOS;
		sscanf(strDecimalNumber, "%lu", pulNum);
		*ppc = p;
	}

	return(rc);
}

bool getCobolName(char **ppc, char *pcCobolName)
{
	bool rc = false;
	char *p = *ppc;
	size_t sIdx = 0;

	if(NULL == p)
		return(false);

	/* skip white space */
	while(*p && isspace(*p))
		p++;

	while(*p && strchr(
		"0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz-", *p))
	{
		if(sIdx < MAX_COBOL_NAME)
		{
			rc = true;
			pcCobolName[sIdx++] = *p++;
		}
		else
		{
			pcCobolName[sIdx] = EOS;
			fprintf(stderr, "COBOL name too long: '%s' (truncated)\n",
				pcCobolName);
			return(false);
		}
	}
	if(rc)
	{
		pcCobolName[sIdx] = EOS;
		*ppc = p;
	}

	return(rc);
}

bool getDMName(char **ppc, char *pcDMName)
{
	bool rc = false;
	char *p = *ppc;
	size_t sIdx = 0;

	if(NULL == p)
		return(false);

	/* skip white space */
	while(*p && isspace(*p))
		p++;

	/* start with letter or '%' (for DM output files) */
	while(*p && strchr(
		"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz%", *p))
	{
		if(sIdx < MAX_DM_NAME)
		{
			rc = true;
			pcDMName[sIdx++] = *p++;
		}
		else
			rc = false;
	}
	while(*p && strchr(
		"0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz_", *p))
	{
		if(sIdx < MAX_DM_NAME)
		{
			rc = true;
			pcDMName[sIdx++] = *p++;
		}
		else
		{
			pcDMName[sIdx] = EOS;
			fprintf(stderr, "DM name too long: '%s' (truncated)\n",
				pcDMName);
			return(false);
		}
	}
	if(rc)
	{
		pcDMName[sIdx] = EOS;
		*ppc = p;
	}

	return(rc);
}


bool getQuotedString(char **ppc, char *pcStr, size_t sLen)
{
	bool rc = true;
	char *p = *ppc;
	size_t sIdx = 0;

	if(NULL == p)
		return(false);

	/* skip white space */
	while(*p && isspace(*p))
		p++;

	if('"' == *p)
		p++;
	else
		return(false);

	while(*p && '"' != *p)
	{
		if(sIdx < sLen)
		{
			pcStr[sIdx++] = *p++;
		}
		else
		{
			pcStr[sIdx] = EOS;
			fprintf(stderr, "quoted string too long: '%s' (truncated)\n",
				pcStr);
			return(false);
		}
	}

	if('"' == *p)
		p++;
	else
		return(false);

	if(rc)
	{
		pcStr[sIdx] = EOS;
		*ppc = p;
	}

	return(rc);
}


static relop getRelopCode(char *strRelop)
{
	int i;
	bool found;

	for(i = 0, found = false;
		!found && 0 != strcmp("", relopTable[i].relopName); i++)
	{
		if(0 == strcmp(strRelop, relopTable[i].relopName))
			found = true;
	}
	if(found)
		return(relopTable[--i].r);
	else
		return(unknownRelop);
}


static bool getRelop(char **ppc, relop *pRelop)
{
	bool rc = false;
	char *p = *ppc;
	char strRelop[MAX_RELOP_LEN + 1];
	size_t sIdx = 0;

	if(NULL == p)
		return(false);

	/* skip white space */
	while(*p && isspace(*p))
		p++;

	while(*p && ',' != *p)
	{
		if(sIdx < MAX_RELOP_LEN)
		{
			rc = true;
			strRelop[sIdx++] = *p++;
		}
		else
		{
			strRelop[sIdx] = EOS;
			fprintf(stderr, "relop too long: '%s' (truncated)\n",
				strRelop);
			return(false);
		}
	}
	if(rc)
	{
		strRelop[sIdx] = EOS;
		*pRelop = getRelopCode(strRelop);
		*ppc = p;
	}

	return(rc);
}


bool getAttribute(char **ppc, attribute *pa)
{
	bool     found;
	char    *p = *ppc;
	size_t   sIdx = 0;
	int      i, j;
	char     strAttName[MAX_ATTRIBUTE_LEN + 1];

	if(NULL == p)
		return(false);

	/* skip white space */
	while(*p && isspace(*p))
		p++;

	while(*p && *p != '(' && !isspace(*p))
	{
		if(sIdx < MAX_ATTRIBUTE_LEN)
			strAttName[sIdx++] = *p++;
		else
		{
			strAttName[sIdx] = EOS;
			fprintf( stderr, "attribute name too long: '%s' (truncated)\n",
																	strAttName );
			return ( false );
		}
	}
	strAttName[sIdx] = EOS;

	if(EOS == strAttName[0])
		return(false);

	/* lookup attribute */
	strlwr(strAttName);
	for(found = false, i = 0;
		!found && 0 != stricmp("", adTable[i].strArgName); i++)
	{
		if(0 == stricmp(strAttName, adTable[i].strArgName))
			found = true;
	}
	if(found)
		i--;
	else
		return(false);

	pa->type = adTable[i].attType;

	if('(' == *p)
		p++;
	else
	   return(false);

	for(j = 0; j < adTable[i].iArgs; j++)
	{
		if(j > 0)
		{
			if(',' == *p)
				p++;
			else
				return(false);
		}
		switch(adTable[i].pArgInfo[j].at)
		{
		case cobolFieldArgType:
			if(!getCobolName(&p,
				(char *) (((char *)pa) + adTable[i].pArgInfo[j].iOffset)))
				return(false);
			break;

		case integerArgType:
			if(!getDecimalNumber(&p,
				(ulong *) (((char *)pa) + adTable[i].pArgInfo[j].iOffset)))
				return(false);
			break;

		case stringArgType:
			/* accept quoted string */
			if(!getQuotedString(&p,
				(char *) (((char *) pa) + adTable[i].pArgInfo[j].iOffset),
					adTable[i].pArgInfo[j].iMaxLen))
				return(false);
			break;

		case tokenArgType:
			if(!getToken(&p,
				(char *) (((char *) pa) + adTable[i].pArgInfo[j].iOffset),
					adTable[i].pArgInfo[j].iMaxLen))
				return(false);
			break;

		case relopArgType:
			if(!getRelop(&p,
				(relop *) (((char *)pa) + adTable[i].pArgInfo[j].iOffset)))
				return(false);
			break;

		default:
			fprintf(stderr, "unrecognized argtype %d\n",
				(int) adTable[i].pArgInfo[j].at);
		}
	}

	if(')' == *p)
		p++;
	else
		return(false);

	*ppc = p;
	return(true);
}


attributeDesc *attributeDescLookup(attributeType type)
{
	bool found;
	int i;

	/* lookup attribute */
	for(found = false, i = 0;
		!found && 0 != strcmp("", adTable[i].strArgName); i++)
	{
		if(type == adTable[i].attType)
			found = true;
	}
	if(found)
		i--;
	else
		return(NULL);
	return(&adTable[i]);
}

static void cfStoreOn(char *pc, char *pcf)
{
	sprintf(pc, "%s%s", pc, pcf);
}

static void ulongStoreOn(char *pc, ulong *pul)
{
	sprintf(pc, "%s%ld", pc, *pul);
}

static void strStoreOn(char *pc, char *pstr)
{
	sprintf(pc, "%s\"%s\"", pc, pstr);
}

static void tokStoreOn(char *pc, char *ptok)
{
	sprintf(pc, "%s%s", pc, ptok);
}

static void relopStoreOn(char *pc, relop *prel)
{
	sprintf(pc, "%s%s", pc, relopTable[(int) *prel].relopName);
}

void attributeStoreOn(attribute *pa, char *p)
{
	attributeDesc *pad;
	int i;

	pad = attributeDescLookup(pa->type);
	strcat(p, pad->strArgName);
	strcat(p, "(");
	for(i = 0; i < pad->iArgs; i++)
	{
		if(0 < i)
			strcat(p, ",");
		(*funcStoreOn[(int) pad->pArgInfo[i].at])(p,
			(void *)((char *) pa + pad->pArgInfo[i].iOffset));
	}
	strcat(p, ")");
}


bool getString(char **ppc, char *pcStr, size_t sLen)
{
	bool rc = false;
	char *p = *ppc;
	size_t sIdx = 0;

	if(NULL == p)
		return(false);

	/* skip white space */
	while(*p && isspace(*p))
		p++;

	while(*p && !isspace(*p))
	{
		if(sIdx < sLen)
		{
			rc = true;
			pcStr[sIdx++] = *p++;
		}
		else
		{
			pcStr[sIdx] = EOS;
			fprintf(stderr, "string too long: '%s' (truncated)\n",
				pcStr);
			return(false);
		}
	}
	if(rc)
	{
		pcStr[sIdx] = EOS;
		*ppc = p;
	}

	return(rc);
}


bool getToken(char **ppc, char *pcTok, size_t sLen)
{
	bool rc = false;
	char *p = *ppc;
	size_t sIdx = 0;

	if(NULL == p)
		return(false);

	/* skip white space */
	while(*p && isspace(*p))
		p++;

	while(*p && strchr(
		"0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz_", *p))
	{
		if(sIdx < sLen)
		{
			rc = true;
			pcTok[sIdx++] = *p++;
		}
		else
		{
			pcTok[sIdx] = EOS;
			fprintf(stderr, "token too long: '%s' (truncated)\n",
				pcTok);
			return(false);
		}
	}
	if(rc)
	{
		pcTok[sIdx] = EOS;
		*ppc = p;
	}

	return(rc);
}

typedef struct
{
	char *strTableName;
	trans *pt;
} transDirectory;

static trans deathTransTable[] =
{
	"D",	"Decreasing",
	"L",	"Level",
	"",	""
};

static trans examTransTable[] =
{
	"2a",		"QUES2a",
	"2b",		"QUES2b",
	"2c",		"QUES2c",
	"2d",		"QUES2d",
	"2e",		"QUES2e",
	"2f",		"QUES2f",
	"2g",		"QUES2g",
	"2h",		"QUES2h",
	"2i",		"QUES2i",
	"2j",		"QUES2j",
	"2k",		"QUES2k",
	"2l",		"QUES2l",
	"3",		"QUES3",
	"4",		"QUES4",
	"5",		"QUES5",
	"6a",		"QUES6a",
	"6b",		"QUES6b",
	"6c",		"QUES6c",
	"6d",		"QUES6d",
	"6e",		"QUES6e",
	"7a",		"QUES7a",
	"7b",		"QUES7b",
	"8",		"QUES8",
	"9a",		"QUES9a",
	"9b",		"QUES9b",
   "misc",  "QUESMISC",
	"blank",	"",
	"",	""
};

static trans examtypeTransTable[] =
{
   "0",  "NOEXAM",
	"1",	"MEDPI",
	"2",	"PMEDPI",
	"3",	"MEDSP",
	"4",	"PMEDSP",
	"",	""
};


static trans flexvTransTable[] =
{
	"16a",	"QUES16a",
	"16b",	"QUES16b",
	"16c",	"QUES16c",
	"17a",	"QUES17a",
	"17b",	"QUES17b",
	"17c",	"QUES17c",
	"18",		"QUES18",
	"19",		"QUES19",
	"20",		"QUES20",
	"21",		"QUES21",
	"blank",	"",
	"",		""
};

/* commented out forms table - rules will now handle it directly 
                                                      IF 9/90 */
/* added form082, form083 and form084 onto the table  IF 6/90 */

/*static trans formTransTable[] =
{
   "174",      "form001",
   "174-VL",	"form002",
   "11007",	   "form003",
   "3151R",	   "form004",
   "3152R",	   "form005",
	"1493R",	   "form006",
	"1797",	   "form007",
	"7581B",	   "form008",
	"7581B-VL",	"form009",
	"3401R",	   "form010",
	"3401R-VL",	"form011",
	"7581A",    "form012",
	"7581A-VL", "form013",
	"BP100",    "form014",
	"15708",    "form015",
	"15575",    "form016",
	"15775",    "form017",
	"15761DE",  "form018",
	"15761DC",  "form019",
	"15761GA",  "form020",
	"15674",    "form021",
	"15576",		"form022",
	"15576C",	"form023",
	"15576S",	"form024",
	"15761MA",	"form025",
	"15761",		"form026",
	"15692",		"form027",
	"15761NY",	"form028",
	"15669",		"form029",
	"15577",		"form030",
	"15675",		"form031",
	"128",		"form032",
	"128-VL",	"form033",
	"129",		"form034",
	"15738",		"form035",
	"15728",		"form036",
	"3509R",		"form037",
	"272R-EC",	"form038",
	"7982",		"form039",
	"157",		"form040",
	"157-VL",	"form041",
	"8258",		"form042",
	"1810R",		"form043",
	"VLI-187",	"form044",
	"1112",		"form045",
	"1112-CR",	"form046",
	"7157",		"form047",
	"303R",		"form048",
	"2980",		"form049",
	"496R",		"form050",
	"VLI-76",	"form051",
	"12942",		"form052",
	"UL-2011",	"form053",
	"15772",		"form054",
	"1341",		"form055",
	"3053",		"form056",
	"1548",		"form057",
	"8218",		"form058",
	"VLI-178",	"form059",
	"7170R",		"form060",
	"13318",		"form061",
	"13817",		"form062",
	"13817VL",	"form063",
	"13817VL-CT",  "form064",
	"13817VL-NC",	"form065",
	"12976",		"form066",
	"174R(I)NY",   "form067",
	"14822",		"form068",
	"473R",		"form069",
	"3508R",		"form070",
	"50R",		"form071",
	"172",		"form072",
	"7016",		"form073",
	"13080",		"form074",
	"VLI-155",	"form075",
	"JHD-12",	"form076",
	"W-9",		"form077",
	"15871",		"form078",
	"VLI-192",	"form079",
	"13515",		"form080",
	"13515VL",	"form081",
   "EKG",      "form082",
   "BPU-101SR","form083",
   "MA FORM",  "form084",
	"",				""
}; */

static trans incomeTransTable[] =
{
	"a",	"10000",
	"b",	"15000",
	"c",	"20000",
	"d",	"25000",
	"e",	"35000",
	"f",	"50000",
	"g",	"75000",
	"h",	"100000",
	"i",	"110000",
	"u",	"0",
	"",	""
};

static trans inforceTransTable[] =
{
	"JH IND",	"JHIND",
	"JH GRP",	"JHGROUP",
	"OT IND",	"OTHIND",                 
	"OT GRP",	"OTHGROUP",
	"",	""
};

static trans lobTransTable[] =
{
	"1",	"PNO",
	"3",	"VLI",
	"4",	"FLEXV",
	"",	""
};

static trans medformTransTable[] =
{
	"OMED",	"PNOMED",
	"OPMED",	"PNOPMED",
	"VMED",	"VLIMED",
	"VPMED",	"VLIPMED",
	"",	""
};

static trans milgradeTransTable[] =
{
	"E-1",	"E1",
	"E-2",	"E2",
	"E-3",	"E3",
	"E-4",	"E4",
	"E-5",	"E5",
	"E-6",	"E6",
	"E-7",	"E7",
	"E-8",	"E8",
	"E-9",	"E9",
	"O-1",	"O1",
	"O-2",	"O2",
	"O-3",	"O3",
	"O-4",	"O4",
	"O-5",	"O5",
	"O-6",	"O6",
	"O-7",	"O7",
	"O-8",	"O8",
	"W-1",	"W1",
	"W-2",	"W2",
	"W-3",	"W3",
	"W-4",	"W4",
	"",	""
};

static trans ownerTransTable[] =
{
	"B",	"BUSINESS",
	"I",	"INSURED",
	"O",	"OTHER",
	"",	""
};

static trans partbTransTable[] =
{
	"1a",		"QUES1a",
	"1b",		"QUES1b",
	"2",		"QUES2",
	"3a",		"QUES3a",
	"3b",		"QUES3b",
	"4",		"QUES4",
   "misc",  "QUESMISC",
	"blank",	"",
	"",		""
};

static trans partcTransTable[] =
{
	"15a",	"QUES15A",
	"15b",	"QUES15B",
	"15c",	"QUES15C",
	"15d",	"QUES15D",
	"15e",	"QUES15E",
	"15f",	"QUES15F",
	"15g",	"QUES15G",
	"15h",	"QUES15H",
	"16a",	"QUES16A",
	"16b",	"QUES16B",
	"17",		"QUES17",
   "misc",  "QUESMISC",
	"",		""
};

static trans physstatTransTable[] =
{
	"B",		"Blank",
	"N",		"None",
	"P",		"Provided",
	"",		""
};

static trans pifTransTable[] =
{
   "0",     "INVALID",
   "1",     "NOTSENT",
   "2",     "NORETURN",
   "3",     "NOHIT",
   "4",     "HIT",
   "5",     "BADREPLY",
	"",		""
};

static trans pnoTransTable[] =
{
	"18a",	"QUES18a",
	"18b",	"QUES18b",
	"18c",	"QUES18c",
	"19a",	"QUES19a",
	"19b",	"QUES19b",
	"19c",	"QUES19c",
	"20",		"QUES20",
	"21",		"QUES21",
	"22",		"QUES22",
	"23",		"QUES23",
	"blank",	"",
	"",		""
};

static trans premTransTable[] =
{
	"A",	"Annual",
	"E",	"MonthlyEC",
	"G",	"MonthlyMA",
	"P",	"MonthlyPC",
	"Q",	"Quart",
	"S",	"SemiAnn",
	"",		""
};

/* added new entries to the relationship table   IF 9/90 */

static trans relTransTable[] =
{
	"Aunt",					"Aunt",
	"Blank",					"Blank",
	"Brother",				"Brother",
   "Brother-in-law",    "Brotherinlaw",
	"Child",    			"Child",
	"Children",    		"Child",
	"Corporation",			"Corp",
	"Cousin",				"Cousin",
	"Daughter",				"Daughter",
   "Daughter-in-law",   "Daughtrinlaw",
	"Father",				"Father",
   "Father-in-law",     "Fatherinlaw",
	"Fiance",				"Fiance",
	"Friend",				"Friend",
	"Granddaughter",		"Granddghtr",
	"Grandfather",			"Grandfthr",
	"Grandmother",			"Grandmthr",
	"Grandson",				"Grandson",
   "Legal Guardian",    "Guardian",
   "Guardian",          "Guardian",
	"Husband",				"Husband",
	"Insured",				"Insured",
	"Estate",         	"InsEstate",
	"Mother",				"Mother",
   "Mother-in-law",     "Motherinlaw",
	"Nephew",				"Nephew",
	"Niece",					"Niece",
	"Other",					"Other",
	"Owner",					"Owner",
	"Parents",				"Parents",
	"Partner",  			"Partner",
	"Partnership",			"Partner",
	"Pension Trust",     "PT",
	"Sister",				"Sister",
   "Sister-in-law",     "Sisterinlaw",
	"Son",					"Son",
   "Son-in-law",        "Soninlaw",
	"Spouse",				"Spouse",
	"Stepdaughter",		"Stepdghtr",
	"Stepfather",		   "Stepfather",
	"Stepmother",   		"Stepmother",
	"Stepson",				"Stepson",
	"Trustee",				"Trustee",
	"Uncle",					"Uncle",
	"Wife",					"Wife",
	"",						""
};

static trans sexTransTable[] =
{
	"F",	"F",
	"M",	"M",
	"",	""
};

static trans specdateTransTable[] =
{
	"Date Written",	"Written",
	"Current Date",	"Current",
	"Other",				"RealDate",
	"",					""
};

static trans stateTransTable[] =
{
	"AB",	"AB",
	"AL",	"AL",
	"AK",	"AK",
	"AZ",	"AZ",
	"AR",	"AR",
	"BC",	"BC",
	"CA",	"CA",
	"CO",	"CO",
	"CT",	"CT",
	"CU",	"CU",
	"CZ",	"CZ",
	"DE",	"DE",
	"DC",	"DC",
	"FL",	"FL",
	"GA",	"GA",
	"GU",	"GU",
	"HI",	"HI",
	"ID",	"ID",
	"IL",	"IL",
	"IN",	"IN",
	"IA",	"IA",
	"KS",	"KS",
	"KY",	"KY",
	"LA",	"LA",
	"ME",	"ME",
	"MD",	"MD",
	"MA",	"MA",
	"MB",	"MB",
	"MI",	"MI",
	"MN",	"MN",
	"MS",	"MS",
	"MO",	"MO",
	"MT",	"MT",
	"MX",	"MX",
	"NB",	"NB",
	"NE",	"NE",
	"NF",	"NF",
	"NS",	"NS",
	"NV",	"NV",
	"NH",	"NH",
	"NJ",	"NJ",
	"NM",	"NM",
	"NY",	"NY",
	"NC",	"NC",
	"ND",	"ND",
	"NT",	"NT",
	"OH",	"OH",
	"OK",	"OK",
	"OR",	"OR",
	"ON",	"ON",
	"OF",	"OF",
	"OP",	"OP",
	"PA",	"PA",
	"PE",	"PE",
	"PI",	"PI",
	"PR",	"PR",
	"PQ",	"PQ",
	"RI",	"RI",
	"SC",	"SC",
	"SD",	"SD",
	"SK",	"SK",
	"TN",	"TN",
	"TX",	"TX",
	"UT",	"UT",
	"VT",	"VT",
	"VA",	"VA",
	"VI",	"VI",
	"WA",	"WA",
	"WV",	"WV",
	"WI",	"WI",
	"WY",	"WY",
	"YI",	"YI",
	"",	 ""
};

static trans vliTransTable[] =
{
	"16a",	"QUES16a",
	"16b",	"QUES16b",
	"16c",	"QUES16c",
	"17a",	"QUES17a",
	"17b",	"QUES17b",
	"17c",	"QUES17c",
	"18",		"QUES18",
	"19",		"QUES19",
	"20",		"QUES20",
	"21",		"QUES21",
	"blank",	"",
	"",		""
};

static trans ynbTransTable[] =
{
	"Y",		"Y",
	"N",		"N",
	"B",     "Blank",
	"",		""
};

/* added new table for military person  IF 6/21/90 */

static trans milperTransTable[] =
{
	"I",     "INSURED",
	"S",     "SPOUSE",
	"B",     "BOTH",
   "O",     "OTHER",
	"",		""
};

static transDirectory transDir[] =
{
	"death",		deathTransTable,
	"exam",		examTransTable,
	"examtype",	examtypeTransTable,
	"flexv",		flexvTransTable,
	"income",	incomeTransTable,
	"inforce",	inforceTransTable,
	"lob",		lobTransTable,
	"medform",	medformTransTable,
	"milgrade",	milgradeTransTable,
	"owner",		ownerTransTable,
	"partb",		partbTransTable,
	"partc",		partcTransTable,
	"physstat",	physstatTransTable,
	"pif",   	pifTransTable,
	"pno",		pnoTransTable,
	"prem",		premTransTable,
	"rel",		relTransTable,
	"sex",		sexTransTable,
	"specdate",	specdateTransTable,
	"state",		stateTransTable,
   "vli",      vliTransTable,
	"ynb",		ynbTransTable,
   "milper",   milperTransTable,
	"",			NULL
};


trans *getTable(char *strTableName)
{
	int i;
	bool found;

	for(i = 0, found = false;
		!found && 0 != stricmp("", transDir[i].strTableName); i++)
	{
		if(0 == stricmp(strTableName, transDir[i].strTableName))
			found = true;
	}
	if(!found)
		return(NULL);
	return(transDir[--i].pt);
}

char *getTableTranslation(char *strTableName, char *strValue, size_t sLen)
{
	int i;
	bool found;
	trans *pt;

	if(0 == sLen)
		return("");

	pt = getTable(strTableName);
	if(NULL == pt)
		return("");

	for(i = 0, found = false;
		!found && 0 != stricmp("", pt[i].strKey); i++)
	{
		if(0 == strnicmp(pt[i].strKey, strValue, sLen))
			found = true;
	}
	if(!found)
		return("");
	else
		return(pt[--i].strTrans);
}
